'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var postcss = _interopDefault(require('postcss'));
var valueParser = _interopDefault(require('postcss-value-parser'));

var index = postcss.plugin("postcss-negative-padding", function () {
	// opts = opts || {};

	var padding = function padding(decl) {
		var values = postcss.list.space(decl.value);

		var properties = ["top", "right", "bottom", "left"];

		if (decl.prop === "padding") {

			if (values.length === 1) {
				values.push(values[0]);
			}
			if (values.length === 2) {
				values.push(values[0]);
			}
			if (values.length === 3) {
				values.push(values[1]);
			}

			properties.forEach(function (side, index) {
				var number = valueParser.unit(values[index]).number;
				var unit = valueParser.unit(values[index]).unit;

				// Create padding property for each side
				if (values[index] === "inherit") {
					decl.before({
						prop: `padding-${side}`,
						value: values[index]
					});
				} else {
					decl.before({
						prop: `padding-${side}`,
						value: Math.abs(number) + unit
					});
				}
			});

			properties.forEach(function (side, index) {
				var number = valueParser.unit(values[index]).number;
				var negative = number < 0;

				// Create margin property for each side if value negative
				if (negative) {
					decl.before({
						prop: `margin-${side}`,
						value: values[index]
					});
				}
			});
		} else {
			var re = /(padding)-(top|right|bottom|left)/i;
			var side = decl.prop.match(re)[2];

			var number = valueParser.unit(decl.value).number;
			var unit = valueParser.unit(decl.value).unit;
			var negative = number < 0;

			if (decl.value === "inherit") {
				decl.before({
					prop: `padding-${side}`,
					value: decl.value
				});
			} else {
				decl.before({
					prop: `padding-${side}`,
					value: Math.abs(number) + unit
				});
			}

			if (negative) {
				decl.before({
					prop: `margin-${side}`,
					value: decl.value
				});
			}
		}

		decl.remove();
	};

	return function (css) {
		var propertyMatch = new RegExp(`^(padding)`);

		css.walkDecls(propertyMatch, function (decl) {
			padding(decl);
		});
	};
});

module.exports = index;
